# Script to generate plots from the state-space assessment model 
# 
# Anders Nielsen <anders@nielsensweb.org> Oct. 2008
oldwd<-getwd()

source('conf/common.R')

############################## read data ##############################

fit.base<-read.fit('baserun/ssass')
fit.current<-read.fit('run/ssass')

setwd('data')

read.ices<-function(filen){
  # Function to read ices data files 
  head<-scan(filen, skip=1, n=7, quiet=TRUE)
  minY<-head[3]
  maxY<-head[4]
  minA<-head[5]
  maxA<-head[6]
  C<-as.matrix(read.table(filen, skip=5, header=F))
  C<-C[,1:(maxA-minA+1)]
  rownames(C)<-minY:maxY
  colnames(C)<-minA:maxA
  return(C)
}

catch.no<-read.ices('canum.dat')
catch.mean.weight<-read.ices('cw.dat')
stock.mean.weight<-read.ices('sw.dat')
prop.mature<-read.ices('mo.dat')
natural.mortality<-read.ices('nm.dat')
catch.no.land<-read.ices('lanum.dat')

setwd(oldwd)

############################## plots ##############################

LO.OK<-file.exists('run/LO.RData')
if(LO.OK){
  load('run/LO.RData')
}

RETRO.OK<-file.exists('run/RETRO.RData')
if(RETRO.OK){
  load('run/RETRO.RData')
}

plots<-function(){
  # SSB plot
  baseplot(fit.base, 'ssb', trans=function(x)x/1000, ylab='SSB (tounsand tonnes)', las=1)
  addlines(fit.current, 'ssb', col='red', trans=function(x)x/1000)

  # Fbar plot
  baseplot(fit.base, 'fbar', ylab=expression(bar(F)[2-4]), las=1)
  addlines(fit.current, 'fbar', col='red')

  # Recruit plot
  baseplot(fit.base, 'R', ylab='Recruits (millions)', trans=function(x)x/1000, las=1)
  addlines(fit.current, 'R', col='red', trans=function(x)x/1000)


  # Residual plot 
  op<-par(mfrow=c(3,1), mar=c(4,4,1,2), mgp=c(2,1,0))
  fleet.names<-c('Total catches', 'IBTS Q1', 'IBTS Q3')
  for(f in 1:3){
    base<-fit.base$res[fit.base$res[,2]==f,]
    cur<-fit.current$res[fit.current$res[,2]==f,]
    scale<-3
    bp(base[,1],base[,3],base[,6], ylim=c(0.5,7.5), xlim=range(fit.base$years), 
       las=1, xlab='year', ylab='Age', main=fleet.names[f], scale=scale)
    points(cur[,1],cur[,3],cex=sqrt(abs(cur[,6]))*scale, col='red', pch=1)
  }
  par(op)


  F<-exp(fit.current$stateEst[-nrow(fit.current$stateEst),-c(1:7)])
  colnames(F)<-c(1:5,'6+')
  rownames(F)<-fit.current$years[-length(fit.current$years)]
  matplot(rownames(F),F,lty=c(1:5,1),col=c(1:4,6,5), type='l', xlab='Year')
  legend('topleft', col=c(1:4,6,5), lty=c(1:5,1), legend=colnames(F), bty='n')

  #nF<-apply(F,1,function(x)x/max(x))
  #bp(as.numeric(colnames(nF)[col(nF)]), row(nF), as.vector(nF), scale=3, ylim=c(.7,6.3))
  #title('Normalized within each year')
  #nF<-apply(F,2,function(x)x/max(x))
  #bp(as.numeric(rownames(nF)[row(nF)]), col(nF), as.vector(nF), scale=3, ylim=c(.7,6.3))
  #title('Normalized within each age')
  
  if(LO.OK){
    # SSB plot 
    baseplot(fit.current, 'ssb', trans=function(x)x/1000, ylab='SSB (tounsand tonnes)', 
             las=1, ci=FALSE, line=FALSE)
    addlines(fit.current, 'ssb', col='red', trans=function(x)x/1000, lwd=2)
    for(i in 1:length(LO)){
      addlines(LO[[i]], 'ssb', col=i+2, trans=function(x)x/1000)
    }
    legend('bottomleft', legend=c('All data', 'w.o. IBTS Q1', 'w.o. IBTS Q3'), 
           lty=rep('solid',3), col=c('red',3,4), bty='n')

    # Fbar plot
    baseplot(fit.current, 'fbar', ylab=expression(bar(F)[2-4]), las=1, ci=FALSE, line=FALSE)
    addlines(fit.current, 'fbar', col='red', lwd=2)
    for(i in 1:length(LO)){
      addlines(LO[[i]], 'fbar', col=i+2)
    }
    legend('bottomleft', legend=c('All data', 'w.o. IBTS Q1', 'w.o. IBTS Q3'), 
           lty=rep('solid',3), col=c('red',3,4), bty='n')

    # Ellipse plot
    civec<-civec(fit.current)
    plot(civec[5:6],xlim=c(civec[5]+c(-10,10)*sqrt(civec[1])),
                    ylim=c(civec[6]+c(-10,10)*sqrt(civec[4])), 
         type='n', xlab=expression(bar(F)[2-4]), ylab='SSB')
    .CI.reg(civec, col='red')
    for(i in 1:length(LO)){
      .CI.reg(LO[[i]]$civec, col=i+2)
    }
    legend('bottomleft', legend=c('All data', 'w.o. IBTS Q1', 'w.o. IBTS Q3'), 
           lty=rep('solid',3), col=c('red',3,4), bty='n')

  }else{
    dummyplot()
    dummyplot()
    dummyplot()
  }

  if(RETRO.OK){
    # SSB plot 
    baseplot(fit.current, 'ssb', trans=function(x)x/1000, ylab='SSB (tounsand tonnes)', 
             las=1, ci=FALSE, line=FALSE)
    addlines(fit.current, 'ssb', col='red', trans=function(x)x/1000, lwd=2)
    for(i in 1:length(RETRO)){
      addlines(RETRO[[i]], 'ssb', col=i+2, trans=function(x)x/1000, with.dot=TRUE)
    }
  
    # Fbar plot
    baseplot(fit.current, 'fbar', ylab=expression(bar(F)[2-4]), las=1, ci=FALSE, line=FALSE)
    addlines(fit.current, 'fbar', col='red', lwd=2)
    for(i in 1:length(RETRO)){
      addlines(RETRO[[i]], 'fbar', col=i+2, with.dot=TRUE)
    }
  }else{
    dummyplot()
    dummyplot()
  }

}


setwd('res')
file.remove(dir(pattern='png$'))
stamp<-gsub('-[[:digit:]]{4}$','',gsub(':','.',gsub(' ','-',gsub('^[[:alpha:]]{3} ','',date()))))
png(filename = paste(stamp,"_%03d.png", sep=''), width = 480, height = 480, 
    units = "px", pointsize = 10, bg = "white")
  plots()    
dev.off()

png(filename = paste("big_",stamp,"_%03d.png", sep=''), width = 1200, height = 1200, 
    units = "px", pointsize = 20, bg = "white")
  plots()    
dev.off()

#pdf(onefile=FALSE, width = 8, height = 8)
#  plots()    
#dev.off()


setwd(oldwd)


############################## Tables ##############################

# SSB and Fbar table 

setwd('res')
file.remove(dir(pattern='html$'))

R<-fit.current$R[,c(1,3,4)]
tsb<-exp(fit.current$logtsb[,c(1,3,4)])
ssb<-exp(fit.current$logssb[,c(1,3,4)])
fbar<-exp(fit.current$logfbar[,c(1,3,4)])

tab1<-cbind(R,tsb,ssb,fbar)
tab1[nrow(tab1),-c(7:9)]<-NA
colnames(tab1)<-c('Recruits', 'Low', 'High', 'TSB', 'Low', 'High', 'SSB', 'Low', 'High', 'F24', 'Low', 'High')
rownames(tab1)<-fit.current$years
xtab(tab1, caption='Table 1. Estimated recruitment, total stock biomass (TBS), spawning stock biomass (SSB), 
                    and average fishing mortality for ages 2 to 4 (F24).', cornername='Year', 
     file=paste(stamp,'_tab1.html',sep=''), dec=c(0,0,0,0,0,0,0,0,0,3,3,3))

tab2<-exp(fit.current$stateEst[,1:7])
rownames(tab2)<-fit.current$years
colnames(tab2)<-c(1:6,'7+')
xtab(tab2, caption='Table 2. Estimated stock numbers.', cornername='Year\\Age', 
     file=paste(stamp,'_tab2.html',sep=''), dec=rep(0,ncol(tab2)))

tab3<-exp(fit.current$stateEst[,-c(1:7)])
tab3<-tab3[-nrow(tab3),]
rownames(tab3)<-fit.current$years[1:nrow(tab3)]
colnames(tab3)<-c(1:5,'6 & 7+')
xtab(tab3, caption='Table 3. Estimated fishing mortalities.', cornername='Year\\Age', 
     file=paste(stamp,'_tab3.html',sep=''), dec=rep(3,ncol(tab3)))

tab4<-exp(fit.current$logscale[,c(1,3,4)])
if(nrow(tab4)>0){
  conffile<-'../run/model.cfg'
  idx1<-grep('Years in which catch data are to be scaled',readLines(conffile))
  idx2<-grep('Define Fbar range',readLines(conffile))
  num<-scan(conffile, skip=idx1, comment.char='#', quiet=TRUE, nlines=idx2-idx1)
  n<-num[1]
  y<-num[2:(n+1)]
  key<-matrix(num[-c(1:(n+1))], nrow=length(y), byrow=TRUE)

  if(!all(apply(key,1,function(x)length(unique(x))==1))){
    tab4<-matrix(tab4[,1][key], nrow=length(y))
    colnames(tab4)<-c(1:6,'7+')
    rownames(tab4)<-y
    xtab(tab4, caption='Table 4. Estimated catch scaling factors', cornername='Year\\Age', 
         file=paste(stamp,'_tab4.html',sep=''), dec=rep(2,ncol(tab4)))
  }else{ # importent special case
    colnames(tab4)<-c('Catch multiplier', 'Low', 'High') 
    rownames(tab4)<-y[key[,1]]
    xtab(tab4, caption='Table 4. Estimated catch scaling factors (same for all ages within each year)', 
         cornername='Year', file=paste(stamp,'_tab4.html',sep=''), dec=rep(2,ncol(tab4)))
  }
}

conffile<-'../run/model.cfg'
idx1<-grep('fishing mortality PARAMETERS', readLines(conffile))
idx2<-grep('Survey q-scaling coefficient', readLines(conffile))
num<-scan(conffile, skip=idx1, comment.char='#', quiet=TRUE, nlines=idx2-idx1)
a<-scan(conffile,, comment.char='#', quiet=TRUE, n=2)
key<-matrix(num, ncol=diff(a)+1, byrow=TRUE)
o<-order(key[key>0])
fleet<-row(key)[key>0][o]
age<-col(key)[key>0][o]
tab5<-cbind(fleet,age,exp(fit.current$logFpar[key[key>0][o],c(1,3,4)])*1000)
colnames(tab5)<-c('Fleet number', 'Age', 'Catchability', 'Low', 'High')
rownames(tab5)<-1:nrow(tab5)
xtab(tab5, caption='Table 5. Estimated catchabilities multiplied by 1000.', 
     cornername='Index',file=paste(stamp,'_tab5.html',sep=''), dec=c(0,0,5,5,5))


tab6<-rbind(c(fit.base$nlogl, fit.base$nopar), 
            c(fit.current$nlogl, fit.current$nopar))
rownames(tab6)<-c('Base', 'Current')
colnames(tab6)<-c('Negative log likelihood','Number of parameters')
if(tab6[1,2]!=tab6[2,2]){
  if(tab6[1,2]<tab6[2,2]){tab6<-tab6[2:1,]}
  tab6<-cbind(tab6,'Degrees of freedom'=c(NA,tab6[1,2]-tab6[2,2]))
  tab6<-cbind(tab6,'P value'=c(NA,1-pchisq(2*(tab6[2,1]-tab6[1,1]),tab6[2,3])))
  xtab(tab6, caption='Table 6. Likelihood values and p-value for the likelihood 
                      ratio test for the model reduction. It is entirely the users 
                      responsibility to ensure that a likelihood ratio test is 
                      meaningful.', 
       cornername='Model',file=paste(stamp,'_tab6.html',sep=''), 
       dec=c(2,0,0,4))
}else{
  xtab(tab6, caption='Table 6. Likelihood values.', 
       cornername='Model',file=paste(stamp,'_tab6.html',sep=''), 
       dec=c(2,0))
}


setwd(oldwd)

